/*
   ; Display the ambient light
   ; =========================

   ;------------------------------------------------------------------------
   ; Author:    Franzi Edo.     The 03/04/98
   ; Modifs:
   ;
   ; Project:   KHEPERA
   ; Goal:              This PRGM launches 3 processes.
   ;                    Process 0: "alive" it toggles the LED 0 every 500 ms.
   ;                    Process 1: it publishes an "IR sensor" association.
   ;                    Process 2: it displays the ambient light sensor values every 1 s.
   ;
   ; This module is (c) of Edoardo Franzi 1992-1998
   ; franzi@k-team.com
   ;
   ;------------------------------------------------------------------------
 */

#include	<sys/kos.h>
#include 	<stdlib.h>
#include 	<stdio.h>

static int32 vIDProcess[3];

/*
 * Process 0
 * ^^^^^^^^^
 *
 * This process changes the state of the LED 0
 * every 500 ms.
 *
 */

static void
process_0 ()
{

  for (;;)
    {
      tim_suspend_task (500);
      var_change_led (0);
    }
}

/*
 * Process 1
 * ^^^^^^^^^
 *
 * This process publishes an association between "IR sens" and a Ptr
 * on the I.R. values.
 *
 */

static void
process_1 ()
{
  int32 status;
  IRSENSOR *sensor;

  sensor = sens_get_pointer ();
  status = tim_define_association ("IR sens", (uint32 *) sensor);
  if (status < 0)
    {
      printf ("It is impossible to generate an IR sens association\r\n");
      exit (0);
    }

  exit (0);
}

/*
 * Process 2
 * ^^^^^^^^^
 *
 * This process displays the I.R. (ambient light) values every 1 s.
 *
 */

static void
process_2 ()
{
  int32 status;
  IRSENSOR *sensor;
  char strout[80];
  uint32 sensBuf[8];
  int i;

  for (;;)
    {
      tim_suspend_task (1000);
      status = tim_find_association ("IR sens");
      if (status < 0)
	{
	  printf ("The IR sens association does not exist!!!\r\n");
	  continue;
	}

      sensor = (IRSENSOR *) status;
      for (i = 0; i < 8; i++)
	sensBuf[i] = sensor->oAmbientLightSensor[i];
      str_cnvt_bin32_dascii (strout, sensBuf, 8);
      printf ("Ambient light sensor values = %s", strout);
    }
}

/*
 * MAIN
 * ====
 *
 * - Initialise the used managers: tim & bios are initialised at the start-up.
 * - Launch all the processes.
 * - Kill the "main". At this moment only the launched tasks are executed.
 *
 */

int
main (void)
{
  int32 status;

  /*
     --------------------------|------------------------------------------------|
   */

  static char prName_0[] = "User 0 process, EF-98                            Rev. 1.00\r\n";
  static char prName_1[] = "User 1 process, EF-98                            Rev. 1.00\r\n";
  static char prName_2[] = "User 2 process, EF-98                            Rev. 1.00\r\n";

  com_reset ();
  var_reset ();
  sens_reset ();
  str_reset ();

  status = install_task (prName_0, 800, process_0);
  if (status == -1)
    exit (0);
  vIDProcess[0] = (uint32) status;

  status = install_task (prName_1, 800, process_1);
  if (status == -1)
    exit (0);
  vIDProcess[1] = (uint32) status;

  status = install_task (prName_2, 800, process_2);
  if (status == -1)
    exit (0);
  vIDProcess[2] = (uint32) status;

  return 0;
}
