/*
   ; Interrupt generation test
   ; =========================

   ;------------------------------------------------------------------------
   ; Author:    Franzi Edo.     The 03/04/98
   ; Modifs:
   ;
   ; Project:   KHEPERA
   ; Goal:              This PRGM launches 2 processes.
   ;                    Process 0: "alive" it toggles the LED 0 every 500 ms.
   ;                    Process 1: it displays the tic-count every 1 S.
   ;
   ;                    An external interruption is installed. It display the
   ;                    number of the interruptions occured.
   ;
   ;                    IMPORTANT INFORMATION ABOUT THE INTERRUPTIONS
   ;                    ---------------------------------------------
   ;
   ;                    IRQ6 is usually reserved for interfacing fast turrets.
   ;                    For this reason, its interruption level is 6 (the highest one!).
   ; 
   ;                    The micro-kernel of Khepera and all the peripherals that can
   ;                    generate an interruption work at level lower than 6
   ;                    (4 for the micro-kernel and 5 for the timers and serial manager).
   ;   
   ;                    68xxx interruption scheme disables all the pending interruptions
   ;                    that occur during the processing of an interruption having a level
   ;                    greater than the pending ones.
   ;                    For this reason to use functions that use lower interruption
   ;                    (ex. the printf) need to readjust the mask of the interruption.
   ;
   ;                    Important note!!! avoid using a manual interruption
   ;                    (putting the line to the GND manually or forcing a
   ;                    continuous GND state).
   ;                    This can crash the kernel with an exception (usually spurius 
   ;                    interruption or bus error).
   ;                    YOU HAVE TO PLACE THE INTERRUPTION WITH A FLIP-FLOP, AND
   ;                    IN THE INTERRUPTION ROUTINE TO CLEAR IT.
   ;
   ; This module is (c) of Edoardo Franzi 1992-1998
   ; franzi@k-team.com
   ;
   ;------------------------------------------------------------------------
 */

#include	<sys/kos.h>
#include 	<stdlib.h>
#include 	<stdio.h>

#define	SET_INTERRUPTION	asm(" and.w #~0x400,SR");

static uint32 vIDProcess[2];

/*
 * Process 0
 * ^^^^^^^^^
 *
 * This process changes the state of the LED 0
 * every 500 ms.
 *
 */

static void
process_0 ()
{

  for (;;)
    {
      tim_suspend_task (500);
      var_change_led (0);
    }
}

/*
 * Process 1
 * ^^^^^^^^^
 *
 * This process displays the tic-count value
 * every 1 s.
 *
 */

static void
process_1 ()
{

  for (;;)
    {
      tim_suspend_task (1000);
      printf ("Number of tic-count = %ld\r\n", tim_get_ticcount ());
    }
}

/*
 * Extension interruption
 * //////////////////////
 *
 * This interruption function increments and dispay a counter.
 *
 */

static void
extInterruption ()
{
  static uint32 vCounter = 0;

/* Disable the IRQ6 interruption and */
/* allow all the k-sys interruptions */

  var_disable_irq ();
  SET_INTERRUPTION

/* Acknowledge the interruption      */
/*                                   */
/* ... your code to do that ...      */

    vCounter++;
  printf ("Counter value = %ld \r\n", vCounter);

/* enable the IRQ6 interruption      */

  var_enable_irq ();
}

/* 
 * MAIN
 * ====
 *
 * - Initialise the used managers: tim & bios are initialised at the start-up.
 * - Launch all the processes.
 * - Kill the "main". At this moment only the launched processes are executed.
 *
 */

int
main (void)
{
  int32 status;

  /*
     --------------------------|------------------------------------------------|
   */

  static char prName_0[] = "User 0 process, EF-98                            Rev. 1.00\r\n";
  static char prName_1[] = "User 1 process, EF-98                            Rev. 1.00\r\n";

  com_reset ();
  var_reset ();

  status = install_task (prName_0, 800, process_0);
  if (status == -1)
    exit (0);
  vIDProcess[0] = (uint32) status;

  status = install_task (prName_1, 800, process_1);
  if (status == -1)
    exit (0);
  vIDProcess[1] = (uint32) status;

/* Now install the interruption function */

  var_set_irq_vector (extInterruption);
  var_enable_irq ();

  return 0;
}
